@@ -22,13 +22,17 @@
 
 """Test the ninjotiff writing."""
 
-import numpy as np
+import colorsys
 import datetime
 import tempfile
-import xarray as xr
+
 import dask.array as da
-import colorsys
+import numpy as np
 import pytest
+import xarray as xr
+from pyninjotiff.ninjotiff import save
+from pyninjotiff.tifffile import TiffFile
+from trollimage.xrimage import XRImage
 
 TIME = datetime.datetime.utcnow()
 DELETE_FILES = True
@@ -53,6 +57,11 @@ class FakeImage(object):
             res = self.data
         return [res.astype(dtype)]
 
+    @property
+    def palette(self):
+        """Return the palette of the image."""
+        return self.data.attrs['palette']
+
 
 class FakeArea(object):
     """Fake area class."""
@@ -66,17 +75,17 @@ class FakeArea(object):
         self.pixel_size_y = (extent[3] - extent[1]) / y_size
 
 
+STEREOGRAPHIC_AREA = FakeArea({'ellps': 'WGS84', 'lat_0': 90.0, 'lat_ts': 60.0, 'lon_0': 0.0, 'proj': 'stere'},
+                              (-1000000.0, -4500000.0, 2072000.0, -1428000.0),
+                              1024, 1024)
+
+
 def test_write_bw():
     """Test saving a BW image.
 
     Reflectances.
     """
-    from pyninjotiff.ninjotiff import save
-    from pyninjotiff.tifffile import TiffFile
-
-    area = FakeArea({'ellps': 'WGS84', 'lat_0': 90.0, 'lat_ts': 60.0, 'lon_0': 0.0, 'proj': 'stere'},
-                    (-1000000.0, -4500000.0, 2072000.0, -1428000.0),
-                    1024, 1024)
+    area = STEREOGRAPHIC_AREA
     scale = 1.0 / 120
     offset = 0.0
     attrs = dict([('resolution', 1050),
@@ -122,12 +131,7 @@ def test_write_bw():
 
 def test_write_bw_inverted_ir():
     """Test saving a BW image."""
-    from pyninjotiff.ninjotiff import save
-    from pyninjotiff.tifffile import TiffFile
-
-    area = FakeArea({'ellps': 'WGS84', 'lat_0': 90.0, 'lat_ts': 60.0, 'lon_0': 0.0, 'proj': 'stere'},
-                    (-1000000.0, -4500000.0, 2072000.0, -1428000.0),
-                    1024, 1024)
+    area = STEREOGRAPHIC_AREA
     scale = 1.0 / 120
     offset = 70.0 / 120
     attrs = dict([('resolution', 1050),
@@ -173,12 +177,7 @@ def test_write_bw_inverted_ir():
 
 def test_write_bw_fill():
     """Test saving a BW image with transparency."""
-    from pyninjotiff.ninjotiff import save
-    from pyninjotiff.tifffile import TiffFile
-
-    area = FakeArea({'ellps': 'WGS84', 'lat_0': 90.0, 'lat_ts': 60.0, 'lon_0': 0.0, 'proj': 'stere'},
-                    (-1000000.0, -4500000.0, 2072000.0, -1428000.0),
-                    1024, 1024)
+    area = STEREOGRAPHIC_AREA
     scale = 1.0 / 120
     offset = 0.0
     attrs = dict([('resolution', 1050),
@@ -229,12 +228,7 @@ def test_write_bw_fill():
 
 def test_write_bw_inverted_ir_fill():
     """Test saving a BW image with transparency."""
-    from pyninjotiff.ninjotiff import save
-    from pyninjotiff.tifffile import TiffFile
-
-    area = FakeArea({'ellps': 'WGS84', 'lat_0': 90.0, 'lat_ts': 60.0, 'lon_0': 0.0, 'proj': 'stere'},
-                    (-1000000.0, -4500000.0, 2072000.0, -1428000.0),
-                    1024, 1024)
+    area = STEREOGRAPHIC_AREA
     scale = 1.0 / 120
     offset = 70.0 / 120
     attrs = dict([('resolution', 1050),
@@ -285,27 +279,10 @@ def test_write_bw_inverted_ir_fill():
 
 def test_write_rgb():
     """Test saving a non-trasparent RGB."""
-    from pyninjotiff.ninjotiff import save
-    from pyninjotiff.tifffile import TiffFile
+    area = STEREOGRAPHIC_AREA
 
-    area = FakeArea({'ellps': 'WGS84', 'lat_0': 90.0, 'lat_ts': 60.0, 'lon_0': 0.0, 'proj': 'stere'},
-                    (-1000000.0, -4500000.0, 2072000.0, -1428000.0),
-                    1024, 1024)
-
-    x_size, y_size = 1024, 1024
-    arr = np.zeros((3, y_size, x_size))
-    radius = min(x_size, y_size) / 2.0
-    centre = x_size / 2, y_size / 2
-
-    for x in range(x_size):
-        for y in range(y_size):
-            rx = x - centre[0]
-            ry = y - centre[1]
-            s = ((x - centre[0])**2.0 + (y - centre[1])**2.0)**0.5 / radius
-            if s <= 1.0:
-                h = ((np.arctan2(ry, rx) / np.pi) + 1.0) / 2.0
-                rgb = colorsys.hsv_to_rgb(h, s, 1.0)
-                arr[:, y, x] = np.array(rgb)
+    fill_value = 0.0
+    arr = create_hsv_color_disk(fill_value)
 
     attrs = dict([('platform_name', 'NOAA-18'),
                   ('resolution', 1050),
@@ -336,7 +313,6 @@ def test_write_rgb():
     data = xr.DataArray(data, coords={'bands': ['R', 'G', 'B']}, dims=[
                         'bands', 'y', 'x'], attrs=attrs)
 
-    from trollimage.xrimage import XRImage
     img = XRImage(data)
 
     with tempfile.NamedTemporaryFile(delete=DELETE_FILES) as tmpfile:
@@ -351,31 +327,30 @@ def test_write_rgb():
                 arr[idx, :, :] * 255).astype(np.uint8))
 
 
-def test_write_rgb_with_a():
-    """Test saving a transparent RGB."""
-    from pyninjotiff.ninjotiff import save
-    from pyninjotiff.tifffile import TiffFile
-
-    area = FakeArea({'ellps': 'WGS84', 'lat_0': 90.0, 'lat_ts': 60.0, 'lon_0': 0.0, 'proj': 'stere'},
-                    (-1000000.0, -4500000.0, 2072000.0, -1428000.0),
-                    1024, 1024)
-
+def create_hsv_color_disk(fill_value):
+    """Create an HSV colordisk."""
     x_size, y_size = 1024, 1024
-    arr = np.zeros((3, y_size, x_size))
+    arr = np.full((3, y_size, x_size), fill_value)
     radius = min(x_size, y_size) / 2.0
     centre = x_size / 2, y_size / 2
-
     for x in range(x_size):
         for y in range(y_size):
             rx = x - centre[0]
             ry = y - centre[1]
-            s = ((x - centre[0])**2.0 + (y - centre[1])**2.0)**0.5 / radius
+            s = ((x - centre[0]) ** 2.0 + (y - centre[1]) ** 2.0) ** 0.5 / radius
             if s <= 1.0:
                 h = ((np.arctan2(ry, rx) / np.pi) + 1.0) / 2.0
                 rgb = colorsys.hsv_to_rgb(h, s, 1.0)
                 arr[:, y, x] = np.array(rgb)
-            else:
-                arr[:, y, x] = np.nan
+    return arr
+
+
+def test_write_rgb_with_a():
+    """Test saving a transparent RGB."""
+    area = STEREOGRAPHIC_AREA
+
+    fill_value = np.nan
+    arr = create_hsv_color_disk(fill_value)
 
     attrs = dict([('platform_name', 'NOAA-18'),
                   ('resolution', 1050),
@@ -406,7 +381,7 @@ def test_write_rgb_with_a():
 
     data = xr.DataArray(data, coords={'bands': ['R', 'G', 'B']}, dims=[
                         'bands', 'y', 'x'], attrs=attrs)
-    from trollimage.xrimage import XRImage
+
     img = XRImage(data)
     with tempfile.NamedTemporaryFile(delete=DELETE_FILES) as tmpfile:
         filename = tmpfile.name
@@ -423,27 +398,10 @@ def test_write_rgb_with_a():
 
 def test_write_rgb_tb():
     """Test saving a non-trasparent RGB with thumbnails."""
-    from pyninjotiff.ninjotiff import save
-    from pyninjotiff.tifffile import TiffFile
+    area = STEREOGRAPHIC_AREA
 
-    area = FakeArea({'ellps': 'WGS84', 'lat_0': 90.0, 'lat_ts': 60.0, 'lon_0': 0.0, 'proj': 'stere'},
-                    (-1000000.0, -4500000.0, 2072000.0, -1428000.0),
-                    1024, 1024)
-
-    x_size, y_size = 1024, 1024
-    arr = np.zeros((3, y_size, x_size))
-    radius = min(x_size, y_size) / 2.0
-    centre = x_size / 2, y_size / 2
-
-    for x in range(x_size):
-        for y in range(y_size):
-            rx = x - centre[0]
-            ry = y - centre[1]
-            s = ((x - centre[0])**2.0 + (y - centre[1])**2.0)**0.5 / radius
-            if s <= 1.0:
-                h = ((np.arctan2(ry, rx) / np.pi) + 1.0) / 2.0
-                rgb = colorsys.hsv_to_rgb(h, s, 1.0)
-                arr[:, y, x] = np.array(rgb)
+    fill_value = 0.0
+    arr = create_hsv_color_disk(fill_value)
 
     attrs = dict([('platform_name', 'NOAA-18'),
                   ('resolution', 1050),
@@ -475,7 +433,6 @@ def test_write_rgb_tb():
     data = xr.DataArray(data, coords={'bands': ['R', 'G', 'B']}, dims=[
                         'bands', 'y', 'x'], attrs=attrs)
 
-    from trollimage.xrimage import XRImage
     img = XRImage(data)
 
     with tempfile.NamedTemporaryFile(delete=DELETE_FILES) as tmpfile:
@@ -568,12 +525,7 @@ def test_write_rgb_tb():
 @pytest.mark.skip(reason="this is no implemented yet.")
 def test_write_rgb_classified():
     """Test saving a transparent RGB."""
-    from pyninjotiff.ninjotiff import save
-    from pyninjotiff.tifffile import TiffFile
-
-    area = FakeArea({'ellps': 'WGS84', 'lat_0': 90.0, 'lat_ts': 60.0, 'lon_0': 0.0, 'proj': 'stere'},
-                    (-1000000.0, -4500000.0, 2072000.0, -1428000.0),
-                    1024, 1024)
+    area = STEREOGRAPHIC_AREA
 
     x_size, y_size = 1024, 1024
     arr = np.zeros((3, y_size, x_size))
@@ -606,7 +558,6 @@ def test_write_rgb_classified():
     data = da.concatenate((data1, datanan, data2), axis=1)
     data = xr.DataArray(data, coords={'bands': ['P']}, dims=['bands', 'y', 'x'], attrs=attrs)
 
-    from trollimage.xrimage import XRImage
     img = XRImage(data)
     with tempfile.NamedTemporaryFile(delete=DELETE_FILES) as tmpfile:
         filename = tmpfile.name
@@ -621,6 +572,60 @@ def test_write_rgb_classified():
         np.testing.assert_allclose(res[:, :, 3] == 0, np.isnan(arr[0, :, :]))
 
 
+def test_write_rgba():
+    """Test saving an RGBA image."""
+    area = STEREOGRAPHIC_AREA
+
+    fill_value = np.nan
+    arr = create_hsv_color_disk(fill_value)
+
+    attrs = dict([('platform_name', 'NOAA-18'),
+                  ('resolution', 1050),
+                  ('polarization', None),
+                  ('start_time', TIME - datetime.timedelta(minutes=55)),
+                  ('end_time', TIME - datetime.timedelta(minutes=50)),
+                  ('level', None),
+                  ('sensor', 'avhrr-3'),
+                  ('ancillary_variables', []),
+                  ('area', area),
+                  ('wavelength', None),
+                  ('optional_datasets', []),
+                  ('standard_name', 'overview'),
+                  ('name', 'overview'),
+                  ('prerequisites', [0.6, 0.8, 10.8]),
+                  ('optional_prerequisites', []),
+                  ('calibration', None),
+                  ('modifiers', None),
+                  ('mode', 'RGBA'),
+                  ('enhancement_history', [{'scale': np.array([1,  1, -1]), 'offset': np.array([0, 0, 1])},
+                                           {'scale': np.array([0.0266347, 0.03559078, 0.01329783]),
+                                            'offset': np.array([-0.02524969, -0.01996642,  3.8918446])},
+                                           {'gamma': 1.6}])])
+
+    kwargs = {'compute': True, 'fill_value': None, 'sat_id': 6300014,
+              'chan_id': 6500015, 'data_cat': 'PPRN', 'data_source': 'SMHI', 'nbits': 8}
+    alpha = np.where(np.isnan(arr[0, :, :]), 0, 1)
+    arr = np.nan_to_num(arr)
+    arr = np.vstack((arr, alpha[np.newaxis, :, :]))
+    data = da.from_array(arr.clip(0, 1), chunks=1024)
+
+    data = xr.DataArray(data, coords={'bands': ['R', 'G', 'B', 'A']}, dims=[
+                        'bands', 'y', 'x'], attrs=attrs)
+
+    img = XRImage(data)
+    with tempfile.NamedTemporaryFile(delete=DELETE_FILES) as tmpfile:
+        filename = tmpfile.name
+        if not DELETE_FILES:
+            print(filename)
+        save(img, filename, data_is_scaled_01=True, **kwargs)
+        tif = TiffFile(filename)
+        res = tif[0].asarray()
+        for idx in range(4):
+            np.testing.assert_allclose(res[:, :, idx], np.round(
+                np.nan_to_num(arr[idx, :, :]) * 255).astype(np.uint8))
+        np.testing.assert_allclose(res[:, :, 3] == 0, alpha == 0)
+
+
 def test_write_bw_colormap():
     """Test saving a BW image with a colormap.
 
@@ -628,12 +633,7 @@ def test_write_bw_colormap():
 
     Reflectances are 0, 29.76, 60, 90.24, 120.
     """
-    from pyninjotiff.ninjotiff import save
-    from pyninjotiff.tifffile import TiffFile
-
-    area = FakeArea({'ellps': 'WGS84', 'lat_0': 90.0, 'lat_ts': 60.0, 'lon_0': 0.0, 'proj': 'stere'},
-                    (-1000000.0, -4500000.0, 2072000.0, -1428000.0),
-                    1024, 1024)
+    area = STEREOGRAPHIC_AREA
     scale = 1.0 / 120
     offset = 0.0
     attrs = dict([('resolution', 1050),
@@ -695,10 +695,7 @@ def test_write_bw_colormap():
         if not DELETE_FILES:
             print(filename)
         save(img, filename, data_is_scaled_01=True, **kwargs)
-        tif = TiffFile(filename)
-        page = tif[0]
-        res = page.asarray(colormapped=False).squeeze()
-        colormap = page.tags['color_map'].value
+        colormap, res = _load_file_values_with_colormap(filename)
 
         assert(len(colormap) == 768)
         assert(np.allclose(colormap[:256], cm_vis))
@@ -707,6 +704,14 @@ def test_write_bw_colormap():
         assert(np.allclose(res[0, ::205], np.array([1,  64, 128, 192, 255])))
 
 
+def _load_file_values_with_colormap(filename):
+    tif = TiffFile(filename)
+    page = tif[0]
+    res = page.asarray(colormapped=False).squeeze()
+    colormap = page.tags['color_map'].value
+    return colormap, res
+
+
 def test_write_ir_colormap():
     """Test saving a IR image with a colormap.
 
@@ -714,12 +719,7 @@ def test_write_ir_colormap():
 
     Temperatures are -70, -40.24, -10, 20.24, 50.
     """
-    from pyninjotiff.ninjotiff import save
-    from pyninjotiff.tifffile import TiffFile
-
-    area = FakeArea({'ellps': 'WGS84', 'lat_0': 90.0, 'lat_ts': 60.0, 'lon_0': 0.0, 'proj': 'stere'},
-                    (-1000000.0, -4500000.0, 2072000.0, -1428000.0),
-                    1024, 1024)
+    area = STEREOGRAPHIC_AREA
     scale = 1.0 / 120
     offset = 70.0 / 120
     attrs = dict([('resolution', 1050),
@@ -787,13 +787,59 @@ def test_write_ir_colormap():
         if not DELETE_FILES:
             print(filename)
         save(img, filename, data_is_scaled_01=True, **kwargs)
-        tif = TiffFile(filename)
-        page = tif[0]
-        res = page.asarray(colormapped=False).squeeze()
-        colormap = page.tags['color_map'].value
+        colormap, res = _load_file_values_with_colormap(filename)
 
         assert(len(colormap) == 768)
         assert(np.allclose(colormap[:256], ir_map))
         assert(np.allclose(colormap[256:512], ir_map))
         assert(np.allclose(colormap[512:], ir_map))
         assert(np.allclose(res[0, ::205], np.array([1,  64, 128, 192, 255])))
+
+
+def test_write_p():
+    """Test saving an image in P mode.
+
+    Values are 0, 1, 2, 3, 4, Palette is black, red, green, blue, gray.
+    """
+    area = STEREOGRAPHIC_AREA
+
+    palette = [np.array((0, 0, 0, 1)),
+               np.array((1, 0, 0, 1)),
+               np.array((0, 1, 0, 1)),
+               np.array((0, 0, 1, 1)),
+               np.array((.5, .5, .5, 1)),
+               ]
+    attrs = dict([('resolution', 1050),
+                  ('polarization', None),
+                  ('platform_name', 'MSG'),
+                  ('sensor', 'seviri'),
+                  ("palette", palette),
+                  ('name', 'msg_cloudtop_height'),
+                  ('level', None),
+                  ('modifiers', ()),
+                  ('start_time', TIME - datetime.timedelta(minutes=85)),
+                  ('end_time', TIME - datetime.timedelta(minutes=80)),
+                  ('area', area),
+                  ('ancillary_variables', [])])
+
+    data = da.tile(da.repeat(da.arange(5, chunks=1024, dtype=np.uint8), 205)[:-1],
+                   1024).reshape((1, 1024, 1024))[:, :1024]
+    data = xr.DataArray(data, coords={'bands': ['P']}, dims=[
+                        'bands', 'y', 'x'], attrs=attrs)
+    kwargs = {'compute': True, 'fill_value': None, 'sat_id': 9000014,
+              'chan_id': 1900015, 'data_cat': 'GPRN', 'data_source': 'SMHI',
+              'physic_unit': 'NONE', "physic_value": "NONE",
+              "description": "NWCSAF Cloud Top Height"}
+
+    img = FakeImage(data)
+    with tempfile.NamedTemporaryFile(delete=DELETE_FILES) as tmpfile:
+        filename = tmpfile.name
+        if not DELETE_FILES:
+            print(filename)
+        save(img, filename, data_is_scaled_01=True, **kwargs)
+        colormap, res = _load_file_values_with_colormap(filename)
+
+        np.testing.assert_array_equal(res[0, ::205], [0, 1, 2, 3, 4])
+        assert(len(colormap) == 768)
+        for i, line in enumerate(palette):
+            np.testing.assert_array_equal(colormap[i::256], (line[:3] * 255).astype(int))
